# -*- coding: utf-8 -*-
##############################################################################
#
#    Copyright (C) 2004-TODAY
#        Norprevención, S.L. (http://www.norprevencion.com) All Rights Reserved
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv,fields
from tools.translate import _
from datetime import date,datetime
from dateutil.relativedelta import *

class hr_holidays(osv.osv):
    """
        Inheritance of class where some methods are overriden due to their lack of functionality
    """

    def _get_working_days(self, date_start,date_end):
        """
            Function that returns a list of working days (weekends removing) between two arbitrary dates
        """
        working_days = 0.0
        year, month, day = (int(x) for x in date_start.split('-'))
        start_date = date(year, month, day)

        year2, month2, day2 = (int(x) for x in date_end.split('-'))
        end_date = date(year2, month2, day2)

        while (start_date <= end_date):
            if (date.weekday(start_date) <= 4):
                working_days+=1
            start_date += relativedelta(days=1)

        return working_days


    def _get_number_of_days(self, cr, uid, date_from, date_to):
        """
         Overriden method that fetchs real working days (using national and local holidays)
        """
        #First, we obtain region associated to partner's company
        current_user = self.pool.get('res.users').browse(cr, uid, uid)
        partner_region_id = current_user.company_id and current_user.company_id.partner_id and current_user.company_id.partner_id.address and current_user.company_id.partner_id.address[0].region and current_user.company_id.partner_id.address[0].region.id or False
        #Then, we have to calculate laboral days
        from_date = date_from[0:date_from.find(" ")]
        to_date = date_to[0:date_to.find(" ")]
        working_days = self._get_working_days(from_date,to_date)
        holiday_sum = 0.0
        if partner_region_id:
            #Finally, we have to substract holiday days
            holiday_facade = self.pool.get('hr.holiday.calendar.event')
            cr.execute("SELECT id FROM hr_holiday_calendar_event where date BETWEEN '%s' and '%s' ORDER BY date" % (date_from,date_to))
            holiday_ids_dict =cr.dictfetchall()
            holiday_ids = [hol['id'] for hol in holiday_ids_dict]
            holiday_days = holiday_facade.browse(cr, uid, holiday_ids)
            for h_day in holiday_days:
                if h_day.scope =='nacional':
                    holiday_sum += 1
                else:
                    for state_id in h_day.state_ids:
                        if partner_region_id == state_id.id:
                            holiday_sum += 1

            final_holiday_days = working_days - holiday_sum
            return final_holiday_days

    def onchange_date_from(self, cr, uid, ids, date_to, date_from):
        result = {}
        if date_to and date_from:
            diff_day = self._get_number_of_days(cr, uid, date_from, date_to)
            result['value'] = {
                'number_of_days_temp': diff_day
            }
            return result
        result['value'] = {
            'number_of_days_temp': 0.0,
        }
        return result

    def check_holidays(self, cr, uid, ids):
        hr_hol_emp_facade = self.pool.get('hr.holidays.employee')
        for record in self.browse(cr, uid, ids):
            if record.holiday_type == 'employee' and record.type == 'remove':
                if record.employee_id and not record.holiday_status_id.limit:
                    request_year = int(record.date_from[0:4])
                    hr_hol_emps = hr_hol_emp_facade.browse(cr, uid, hr_hol_emp_facade.search(cr, uid, [('employee_id', '=', record.employee_id.id), ('year','=', request_year)]))
                    hr_hol_emp = hr_hol_emps and hr_hol_emps[0] or False
                    if hr_hol_emp:
                        leaves_rest = hr_hol_emp.available_holidays + hr_hol_emp.pending_prev_year_holidays
                        if leaves_rest < record.number_of_days_temp:
                            raise osv.except_osv(_('Warning!'),_('You cannot validate leaves for employee %s: too few remaining days (%s).') % (record.employee_id.name, leaves_rest))
                    else:
                        raise osv.except_osv(_('Warning!'),_('Please, first define some holiday period for this employee: %s and year: %s. Go to Human Resources ->Configuration->Holidays->Holidays per Employee') % (record.employee_id.name, request_year))
        return True
    

    _columns = {
        #'company_id': fields.many2one('res.company', 'Company'),
        'company_id': fields.related('employee_id', 'company_id', type='many2one', relation='res.company', string='Company', store=True, readonly=True),
    }
    _inherit = "hr.holidays"

hr_holidays()