# -*- coding: utf-8 -*-
##############################################################################
#
#    Copyright (C) 2004-TODAY
#        Norprevención, S.L. (http://www.norprevencion.com) All Rights Reserved
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

"""
ICS files import wizard for OpenERP
"""

__author__ = "Alberto Luengo Cabanillas (NorPrevencion)"


from osv import osv, fields
from tools.translate import _
import unicodedata
import urllib,urllib2
import base64
import re


SPANISH_REGIONS_DICT = {}
SPANISH_REGIONS_DICT[unicodedata.normalize('NFKD', unicode('Cataluña','utf-8')).encode('ASCII', 'ignore')]='Catalunya',
SPANISH_REGIONS_DICT['Cataluna']='Catalunya',
SPANISH_REGIONS_DICT['Andalucia'] = 'Andalucía',
SPANISH_REGIONS_DICT['Aragon'] = 'Aragón',
SPANISH_REGIONS_DICT['Asturias'] = 'Asturias',
SPANISH_REGIONS_DICT['Baleares'] = 'Balears, Illes',
SPANISH_REGIONS_DICT['Pais Vasco'] = 'Euskadi',
SPANISH_REGIONS_DICT['Euskadi'] ='Euskadi',
SPANISH_REGIONS_DICT['Canarias'] = 'Islas Canarias',
SPANISH_REGIONS_DICT['Cantabria'] ='Cantabria',
SPANISH_REGIONS_DICT['Castilla La Mancha'] ='Castilla La Mancha',
SPANISH_REGIONS_DICT['Castilla-La Mancha'] ='Castilla La Mancha',
SPANISH_REGIONS_DICT[unicodedata.normalize('NFKD', unicode('Castilla y León','utf-8')).encode('ASCII', 'ignore')] ='Castilla y León',
SPANISH_REGIONS_DICT['Extremadura'] ='Extremadura',
SPANISH_REGIONS_DICT['Galicia'] = 'Galicia',
SPANISH_REGIONS_DICT['Madrid'] = 'Madrid',
SPANISH_REGIONS_DICT['Murcia'] ='Múrcia',
SPANISH_REGIONS_DICT['Navarra'] ='Navarra',
SPANISH_REGIONS_DICT['La Rioja'] ='La Rioja',
SPANISH_REGIONS_DICT['Valencia'] = 'País Valencià',
SPANISH_REGIONS_DICT['Ceuta'] ='Ceuta',
SPANISH_REGIONS_DICT['Melilla'] ='Melilla',

class wizard_import_ics_calendar(osv.osv_memory):
    """
        Wizard for importing ICS files
    """


    def action_import_ics_calendar(self, cr, uid, ids, *args):
        """
        Method that reads an ICS file and builds calendar events in OpenERP
        """
        holiday_event_facade = self.pool.get('hr.holiday.calendar.event')
        country_region_facade = self.pool.get('res.country.region')
        view_facade = self.pool.get('ir.ui.view')
        created_event_ids = []
        vals = {}
        flag = 0
        for wiz in self.browse(cr, uid, ids):
            hol_reads = holiday_event_facade.read(cr, uid, holiday_event_facade.search(cr, uid, []),['scope', 'date'])

            if wiz.check_selection_mode == 'url':
                calendar = urllib.urlopen(wiz.ics_url)
                if calendar.url[len(calendar.url)-3:] != 'ics':
                    raise osv.except_osv(_('Warning!'), _('Entered URL is not valid!'))
                ics_file_content = calendar.read()
            else:
                #Estamos subiendo un fichero .ics
                ics_file_content = base64.b64decode(wiz.input_ics_file)
            data_to_read = ics_file_content.split("\n")
            list_festivos = []
            for line in data_to_read:
                if len(line) == 0:
                    continue
                if (line[0:12]=='BEGIN:VEVENT'): # Comienzo de evento festivo
                    festivo = []
                    flag = 1

                if ((line[0:7]=='DTSTART') and flag==1):#Metemos la fecha del evento
                    #Cogemos el valor que va desde los dos puntos al final
                    startDate = line[line.find(":")+1:len(line)-1]
                    festivo.insert(0, startDate)

                if ((line[0:7]=='SUMMARY') and flag==1):#Metemos la descripcion del evento
                    summary_field = unicodedata.normalize('NFKD', unicode(line[8:(len(line)-1)],'utf-8')).encode('ASCII', 'ignore')
                    festivo.append(summary_field)
                    found = False
                    list_states= []
                    #Busco el nombre de la comunidad autonoma en el campo 'SUMMARY' de la linea procesada...
                    for key in SPANISH_REGIONS_DICT.keys():
                        if re.search(key, summary_field, re.M|re.I):
                            region_id = country_region_facade.search(cr, uid, [('name','=', SPANISH_REGIONS_DICT[key][0])])
                            region_id = region_id and region_id[0] or False
                            if region_id:
                                list_states.append(region_id)
                                found = True
                    if not found:
                        festivo.append('nacional')
                        festivo.append(False)
                    else:
                        festivo.append('autonomico')
                        festivo.append(list_states)

                if ((line[0:10]=='END:VEVENT') and flag==1): #Fin de evento festivo
                    flag = 0
                    list_festivos.append(festivo)
            
            #Una vez recorrido el documento, creamos los festivos...
            for festivo in list_festivos:
                formatted_date = festivo[0][0:4]+"-"+festivo[0][4:6]+"-"+festivo[0][6:8]+" 00:00:00"
                vals['date'] = formatted_date
                vals['name'] = festivo[1]
                vals['scope'] = festivo[2]
                vals['duration'] = 8.0
                vals['country_id'] = wiz.country_id.id
                vals['company_id'] = 1
                if festivo[2] != 'nacional':
                    vals['state_ids'] = [(6,0,festivo[3])]
                else:
                    vals['state_ids'] = False
                vals['allday'] = True
                if not hol_reads: #Si no existe ningun registro en BD, lo creamos
                    try:
                        event_id = holiday_event_facade.create(cr, uid, vals)
                    except Exception, e:
                        msg = _("Operation could not be completed due to following error: '%s'. Please, check your file or URL and try again."%(e))
                        raise osv.except_osv(_('Warning!'), _(msg))
                    created_event_ids.append(event_id)
                else:
                    for hol_pair in hol_reads: #Examinamos que no haya registros que coincidan en fecha y alcance
                        if formatted_date == hol_pair['date'] and hol_pair['scope'] == vals['scope']:
                            found = True
                    if not found:
                        event_id = holiday_event_facade.create(cr, uid, vals)
                        created_event_ids.append(event_id)
            
            if not created_event_ids:
                raise osv.except_osv(_('Warning!'), _('All holidays present in this file have been already included. Nothing has been done...'))
            else:
                #Por último, devolvemos la vista correspondiente...
                holiday_event_list_view_id = view_facade.search(cr, uid, [('model', '=', 'hr.holiday.calendar.event'), ('type','=','tree')],order="id DESC")[0]
                holiday_event_form_view_id = view_facade.search(cr, uid, [('model', '=', 'hr.holiday.calendar.event'), ('type','=','form')],order="id DESC")[0]
                holiday_event_calendar_view_id = view_facade.search(cr, uid, [('model', '=', 'hr.holiday.calendar.event'), ('type','=','calendar')],order="id DESC")[0]
                
                return {
                    'name' : _('Created Holiday Calendar'),
                    'type' : 'ir.actions.act_window',
                    'res_model' : 'hr.holiday.calendar.event',
                    'view_type' : 'form',
                    'view_mode' : 'calendar, tree,form',
                    'domain' : "[('id', 'in', %s)]" % (created_event_ids),
                    'view_id' : False,
                    'views': [(holiday_event_calendar_view_id, 'calendar'), (holiday_event_list_view_id, 'tree'), (holiday_event_form_view_id, 'form'), (False, 'graph')],
                }


    _name = 'wizard.import.ics.calendar'
    _description = "ICS calendar files import Wizard"

    _columns = {
        'check_selection_mode': fields.selection([('url', 'URL'),('upload', 'Upload file')], 'Mode'),
        'input_ics_file': fields.binary('ICS file', filters="*.ics"),
        'input_ics_file_name': fields.char('File name', size=256),
        'ics_url': fields.char('ICS file web address', size=256),
        'country_id': fields.many2one('res.country','Country')
    }
    

wizard_import_ics_calendar()



